/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "../lib/defs.h"
#include "commit.h"

FILE *fh;

char *convifname(char *name);

void print_ports(char *protocol,char *port,char *icmp)
{
	int p0,p1;

	/* source ip ports */
	if( (!strncmp(protocol,"tcp",3)) 
	 || (!strcmp(protocol,"udp")) ){
		if(strcmp(port,"ALL")){
			ports_getportsbyname(PORTS_ALL,port,&p0,&p1);
			if(p1){
				fprintf(fh,"%d:%d ",p0,p1);
			} else {
				fprintf(fh,"%d ",p0);
			}
		}
	} else if( (!strcmp(protocol,"icmp")) ){
		if(strcmp(icmp,"ALL")){
			fprintf(fh,"%s ",icmp);
		}
	}
}

void print_ipaddr(char *type,char *group)
{
	fprintf(fh,"%s %s ",type,convifname(groups_getgroupvalbyname(GROUPS_ALL,group)));
}

void print_protocol(char *protocol)
{
	if( (!strcmp(protocol,"tcp")) ){
		fprintf(fh,"-p TCP ");
	} else if( (!strcmp(protocol,"tcpsyn")) ){
		fprintf(fh,"-p TCP -y ");
	} else if( (!strcmp(protocol,"udp")) ){
		fprintf(fh,"-p UDP ");
	} else if( (!strcmp(protocol,"icmp")) ){
		fprintf(fh,"-p ICMP ");
	} else if( (!strcmp(protocol,"frag")) ){
		fprintf(fh,"-f ");
	}
}

int filternolist(char *filter,char *action,char *module,char *file,char *help)
{
	fprintf(fh,"      %s -r %s\n",MODPROBE,module);
	return(0);
}
int filterlist(char *filter,char *action,char *module,char *file,char *help)
{
	if(!strcmp(action,"ON")){
		fprintf(fh,"      %s %s\n",MODPROBE,file);
	} else {
		fprintf(fh,"      %s -r %s\n",MODPROBE,module);
	}
	return(0);
}

char *convifname(char *name)
{
	static char envname[60];
	int i;
	for(i=0;i<59;i++){
		switch(name[i]){
		case ':':
		case '-':
			envname[i]='_';
			break;
		default:
			envname[i]=name[i];
		}
		if(!name[i])
			break;
	}
	name[i]=0;
	return(envname);
}

int listif(char *name,char *type,char *val)
{
	/* IP address */
	fprintf(fh,"      %s=`%s %s`/32\n",convifname(name),IFIP,name);
	if( !strncmp(name,"eth",3)
	 || !strncmp(name,"lo",2) ){
		/* Network address */
		fprintf(fh,"      %s_net=`%s %s`/`%s %s`\n",convifname(name),IFNW,name,IFNM,name);
		/* Broadcast address */
		fprintf(fh,"      %s_bcs=`%s %s`/32\n",convifname(name),IFBC,name);
	}
	return(0);
}

int mkrulelist(struct RULE *rule)	
{
	/* print name of action */
	if(!strcmp(rule->action,"MASQ"))
		fprintf(fh,"-j MASQ ");
	if(!strcmp(rule->action,"ACCEPT"))
		fprintf(fh,"-j ACCEPT ");
	if(!strcmp(rule->action,"REJECT"))
		fprintf(fh,"-j REJECT ");
	if(!strcmp(rule->action,"REJECTLOG"))
		fprintf(fh,"-j REJECT ");
	if(!strcmp(rule->action,"DENY"))
		fprintf(fh,"-j DENY ");
	if(!strcmp(rule->action,"DENYLOG"))
		fprintf(fh,"-j DENY ");
	/* optional iface field */
	if(strcmp(rule->iface,"ALL")){
		fprintf(fh,"-i %s ",rule->iface);
	}
	
	/* optional protocol field */
	print_protocol(rule->protocol);

	/* source ip address */
	print_ipaddr("-s",rule->srcip);

	/* source ip ports */
	print_ports(rule->protocol,rule->srcport,rule->icmptype);

	/* destination ip address */
	print_ipaddr("-d",rule->dstip);

	/* destination ip ports */
	print_ports(rule->protocol,rule->dstport,rule->icmpcode);
		
	/* optional logging flag */
	if( (!strcmp(rule->action,"DENYLOG"))
	  ||(!strcmp(rule->action,"REJECTLOG")) ){
		fprintf(fh,"-l ");
	}

	fprintf(fh,"\n");
	return(0);
}

int mkinputrulelist(struct RULE *rule)
{
	fprintf(fh,"      %s -A input ",IPCHAINS);
	mkrulelist(rule);
	return(0);
}
int mkoutputrulelist(struct RULE *rule)
{
	fprintf(fh,"      %s -A output ",IPCHAINS);
	mkrulelist(rule);
	return(0);
}
int mkforwardrulelist(struct RULE *rule)
{
	fprintf(fh,"      %s -A forward ",IPCHAINS);
	mkrulelist(rule);
	return(0);
}

char *stripnetmask(char *addr)
{
	char *sptr;
	if(sptr=index(addr,'/'))
		*sptr=0;
	return(addr);
}
char *stripalias(char *iface)
{
	char *sptr;
	if(sptr=index(iface,':'))
		*sptr=0;
	return(iface);
}
char redirif[24];
char rediriface[24];
int redirport;
int listtargetinput(struct REDIR_TARGET *target)
{
	int rp;
	ports_getportsbyname(PORTS_ALL,target->port,&rp,NULL);

	fprintf(fh,"      %s mfw -A -m %d -r %s %d\n",
		IPMASQADM,target->mark,
		stripnetmask(groups_getgroupvalbyname(GROUPS_ALL,target->host)),
		rp);
	return(0);
}
int listredirinput(struct REDIR *redir)
{
	ports_getportsbyname(PORTS_ALL,redir->port,&redirport,NULL);

	/* mark the packet for redirection */
	fprintf(fh,"      #Allow redirection for IF:%s PORT:%s (%d)\n",
		redir->iface,redir->port,redirport);
	fprintf(fh,"      %s -A input -j ACCEPT -i %s -p TCP -s 0.0.0.0/0 -d %s %d -m %d\n",
		IPCHAINS,
		stripalias(redir->iface),
		convifname(groups_getgroupvalbyname(GROUPS_ALL,redir->iface)),
		redirport,
		redir->mark);

	/* redirect the packets to these destinations */
	redir_targetenum(redir->mark,listtargetinput);
	return(0);
}
int listtargetoutput(struct REDIR_TARGET *target)
{
	fprintf(fh,"      %s -A output -j ACCEPT -p TCP -s %s %d -d %s %d\n",
		IPCHAINS,
		rediriface,
		redirport,
		convifname(groups_getgroupvalbyname(GROUPS_ALL,target->host)),
		redirport);
	return(0);
}
int listrediroutput(struct REDIR *redir)
{
	strcpy(rediriface,convifname(groups_getgroupvalbyname(GROUPS_ALL,redir->iface)));
	ports_getportsbyname(PORTS_ALL,redir->port,&redirport,NULL);

	fprintf(fh,"      #Allow traffic out from redirected port\n");
	fprintf(fh,"      %s -A output -j ACCEPT -p TCP -i %s -s %s %d -d 0.0.0.0/0\n",
		IPCHAINS,
		stripalias(redir->iface),
		rediriface,
		redirport);

	redir_targetenum(redir->mark,listtargetoutput);
	return(0);
}
int listtargetforward(struct REDIR_TARGET *target)
{
	fprintf(fh,"      %s -A forward -j MASQ -i %s -p TCP -s %s %d\n",
		IPCHAINS,
		stripalias(redirif),
		convifname(groups_getgroupvalbyname(GROUPS_ALL,target->host)),
		redirport);
	return(0);
}
int listredirforward(struct REDIR *redir)
{
	ports_getportsbyname(PORTS_ALL,redir->port,&redirport,NULL);
	strcpy(rediriface,convifname(groups_getgroupvalbyname(GROUPS_ALL,redir->iface)));
	strcpy(redirif,redir->iface);
	redir_targetenum(redir->mark,listtargetforward);
	return(0);
}

int listpolroutes(struct POLROUTING *pr)
{
	fprintf(fh,"      %s -A POLROUTE ",IPCHAINS);
	print_protocol(pr->protocol);
	print_ipaddr("-s",pr->srcip);
	print_ports(pr->protocol,pr->srcport,pr->icmptype);
	print_ipaddr("-d",pr->dstip);
	print_ports(pr->protocol,pr->dstport,pr->icmpcode);
	fprintf(fh,"-m %d ",pr->mark);
	fprintf(fh,"-j RETURN ");
	fprintf(fh,"\n");
}
int listpolrouters(struct POLROUTER *pr)
{
	fprintf(fh,"      %s route flush table %d >/dev/null 2>/dev/null\n",
		IPTOOL,
		pr->mark
	);
	fprintf(fh,"      %s route add table %d via %s\n",
		IPTOOL,
		pr->mark,
		stripnetmask(groups_getgroupvalbyname(GROUPS_ALL,pr->router))
	);
	return(0);	
}
int listpolrouting(struct POLROUTING *pr)
{
	fprintf(fh,"      %s rule delete fwmark %x >/dev/null 2>/dev/null\n",
		IPTOOL,
		pr->mark
	);

	fprintf(fh,"      %s rule add fwmark %x table %d priority %d\n",
		IPTOOL,
		pr->mark,
		pr->mark,
		pr->mark
	);

	return(0);
}
int main(int argc,char *argv[])
{
	char cmd[256];

	if(html_system(CHECK_LOGIN)){
		return(0);
	}

	if(!(fh=fopen(FWT_RC_FILE,"w"))){
		system(RELOAD_COMMIT);
		return(0);
	}

	fprintf(fh,"#!/bin/sh\n");
	fprintf(fh,"#This file was machine-generated by FWT application\n");
	fprintf(fh,"#Do not modify!\n");
	fprintf(fh,"\n"); 
	fprintf(fh,"case \"$1\" in\n"); 
	fprintf(fh,"   start)\n"); 
	fprintf(fh,"      echo \"Starting Firewall\"\n"); 
	fprintf(fh,"      echo 1 >%s\n",IP_FORWARDING);
	fprintf(fh,"      #all hw interfaces needs to be evaluated at runtime\n");
	fprintf(fh,"      #note: ':' is not valid in a variablename\n");
	groups_enum(GROUPS_IF,listif);

	/* policyrouters */
	if(pr_num()){
		fprintf(fh,"\n");
		fprintf(fh,"      #Policy routers and routing tables\n");
		pr_enum(listpolrouters);
	}

	if(pr_numroutes(-1)){
		fprintf(fh,"\n");
		fprintf(fh,"      #Policy routing: route matching packets via table\n");
		fprintf(fh,"      #Note that fwmark is specified in HEX\n");
		pr_enumroutes(-1,listpolrouting);
	}

	fprintf(fh,"      #input rules\n");
	fprintf(fh,"      %s -F input\n",IPCHAINS);
	fprintf(fh,"      %s mfw -F\n",IPMASQADM);
	/* input rules to mark policy routed packets */
	/* only do this if there is anything to policy route */
	if(pr_numroutes(-1)){
		fprintf(fh,"\n");
		fprintf(fh,"      #Policy routes\n");
		fprintf(fh,"      %s -F POLROUTE >/dev/null 2>/dev/null\n",IPCHAINS);
		fprintf(fh,"      %s -N POLROUTE >/dev/null 2>/dev/null\n",IPCHAINS);
		pr_enumroutes(-1,listpolroutes);
		fprintf(fh,"      %s -A input -j POLROUTE\n",IPCHAINS);
	}	

	fprintf(fh,"      #Redirection rules for Input\n");
	redir_enum(listredirinput);
	fprintf(fh,"      #End of redirection rules\n");
	rules_enum(RULES_IN,mkinputrulelist);
	fprintf(fh,"      %s -A input -j DENY -l\n",IPCHAINS);
	fprintf(fh,"      #output rules\n"); 
	fprintf(fh,"      %s -F output\n",IPCHAINS);
	fprintf(fh,"      #Redirection rules for Output\n");
	redir_enum(listrediroutput);
	fprintf(fh,"      #End of redirection rules\n");
	rules_enum(RULES_OU,mkoutputrulelist);
	fprintf(fh,"      %s -A output -j DENY -l\n",IPCHAINS);
	fprintf(fh,"      #forward rules\n");
	fprintf(fh,"      %s -F forward\n",IPCHAINS);
	fprintf(fh,"      #Redirection rules for Forward\n");
	redir_enum(listredirforward);
	fprintf(fh,"      #End of redirection rules\n");
	rules_enum(RULES_FW,mkforwardrulelist);
	fprintf(fh,"      %s -A forward -j DENY -l\n",IPCHAINS);
	fprintf(fh,"      #filters\n");
	filter_enum(filterlist);
	fprintf(fh,"      ;;\n");
	fprintf(fh,"   stop)\n");
	fprintf(fh,"      echo Shutting down firewall\n");
	fprintf(fh,"      echo 0 >%s\n",IP_FORWARDING);
	fprintf(fh,"      %s mfw -F\n",IPMASQADM);
	fprintf(fh,"      %s -F input\n",IPCHAINS);
	fprintf(fh,"      %s -A input -j ACCEPT -i lo\n",IPCHAINS);
	fprintf(fh,"      %s -A input -j DENY\n",IPCHAINS);
	fprintf(fh,"      %s -F output\n",IPCHAINS);
	fprintf(fh,"      %s -A output -j ACCEPT -i lo\n",IPCHAINS);
	fprintf(fh,"      %s -A output -j DENY\n",IPCHAINS);
	fprintf(fh,"      %s -F forward\n",IPCHAINS);
	fprintf(fh,"      %s -A forward -j DENY\n",IPCHAINS);
	fprintf(fh,"      #filters\n");
	filter_enum(filternolist);
	fprintf(fh,"      ;;\n");
	fprintf(fh,"   stop)\n");
	fprintf(fh,"      %s -F input\n",IPCHAINS);
	fprintf(fh,"      %s -A input -j ACCEPT\n",IPCHAINS);
	fprintf(fh,"      %s -F output\n",IPCHAINS);
	fprintf(fh,"      %s -A output -j ACCEPT\n",IPCHAINS);
	fprintf(fh,"      %s -F forward\n",IPCHAINS);
	fprintf(fh,"      %s -A forward -j ACCEPT\n",IPCHAINS);
	fprintf(fh,"      ;;\n");
	fprintf(fh,"   *)\n");
	fprintf(fh,"      echo \"Usage <fwt> {start|stop}\"\n"); 
	fprintf(fh,"      exit 10\n");
	fprintf(fh,"esac\n"); 
	fclose(fh);
	chmod(FWT_RC_FILE,S_IRUSR|S_IWUSR|S_IXUSR);
	/* run the new config */
	sprintf(cmd,"%s start >/dev/null 2>/dev/null",FWT_RC_FILE);
	system(cmd);
	/* reload commit page */
	html_system(RELOAD_RULES);
	return(0);
}

