/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <net/if.h>
#include <dirent.h>
#include "../lib/defs.h"
#include "ifcfg.h"

int numaliases;
int countalias(char *name,char *type,char *val)
{
	if(index(name,':'))
		numaliases++;
	return(0);
}
int listdeletealias(char *name,char *type,char *val)
{
	if(index(name,':'))
		printf("<option value=\"%s\">%s\n",name,name);
	return(0);
}

int inittable(void)
{
	printf("<tr>");
	printf("<th>Interface</th>");
	printf("<th>Address</th>");
	printf("<th>NetMask</th>");
	printf("<th>Status</th>");
	printf("<th>OnBoot</th>");
	printf("</tr>\n");
	return(0);
}
int listtable(char *name,char *type,char *val)
{
	struct IFDATA ifd;

	ifcfg_getifdata(name,&ifd);
	printf("<tr>");
	printf("<td>%s</td>\n",name);
	printf("<td>%d.%d.%d.%d</td>",ifd.a0,ifd.a1,ifd.a2,ifd.a3);
	printf("<td>%s</td>",ifcfg_masktostring(ifd.mask));
	if(index(name,':')){
		printf("<td>%s</td>",(ifd.flags&IFF_UP)?"up":"down");
		printf("<td>%s</td>",(ifd.onboot)?"up":"down");
	} else {
		printf("<td>%s</td>",(ifd.flags&IFF_UP)?"UP":"DOWN");
		printf("<td>%s</td>",(ifd.onboot)?"UP":"DOWN");
	}
	printf("</tr>\n");
	return(0);
}
	
int listcreatealias(char *name,char *type,char *val)
{
	if(!strncmp(name,"lo",2)) /* loopback cant have aliases */
		return(0);
	if(strncmp(name,"eth",3)) /* only ehternet can have aliases */
		return(0);
	if( index(name,':') ) /* cant create alias on alias if */
		return(0);
	printf("<option value=\"%s\">%s\n",name,name);
	return(0);
}
int listeditif(char *name,char *type,char *val)
{
	if(strncmp(name,"eth",3)) /* only ethernet and aliases can be changed */
		return(0);
	printf("<option value=\"%s\">%s\n",name,name);
	return(0);
}

int listshowif(char *name,char *type,char *val)
{
	printf("<option value=\"%s\">%s\n",name,name);
	return(0);
}

int main(int argc,char *argv[])
{
	if(html_system(CHECK_LOGIN)){
		exit(10);
	}

	html_title("FWT Network Configuration");
	printf("<BODY>\n");
	printlinks(IFCFG_LINKS,NULL);
	printf("<H3>Network Configuration</H3>\n");

	/* examine iface */
	printf("<FORM ACTION=\"%s\" METHOD=POST>\n",IFCFG_SHOWIF);
	printf("<select name=\"show_if\">\n");
	groups_enum(GROUPS_IF,listshowif);
	printf("</select>\n");
	printf("<input type=\"submit\" value=\"EXAMINE INTERFACE\">\n");
	printf("<br></form>\n");

	/* create list of interfaces to edit */
	printf("<FORM ACTION=\"%s\" METHOD=POST>\n",IFCFG_EDITIF);
	printf("<select name=\"edit_if\">\n");
	groups_enum(GROUPS_IF,listeditif);
	printf("</select>\n");
	printf("<input type=\"submit\" value=\"EDIT INTERFACE\">\n");
	printf("<br></form>\n");

	/* create list of interfaces to create */
	printf("<FORM ACTION=\"%s\" METHOD=POST>\n",IFCFG_CREATEALIAS);
	printf("<select name=\"create_alias\">\n");
	groups_enum(GROUPS_IF,listcreatealias);
	printf("</select>\n");
	printf("<input type=\"submit\" value=\"CREATE ALIAS\">\n");
	printf("<br></form>\n");

	/* delete alias, if there are any */
	numaliases=0;
	groups_enum(GROUPS_IF,countalias);
	if(numaliases){
		printf("<FORM ACTION=\"%s\" METHOD=POST>\n",IFCFG_DELETEALIAS);
		printf("<select name=\"delete_alias\">\n");
		groups_enum(GROUPS_IF,listdeletealias);
		printf("</select>\n");
		printf("<input type=\"submit\" value=\"DELETE ALIAS\">\n");
		printf("<br></form>\n");
	}

	/* create table */
	printf("<table border>\n");
	printf("<caption><strong>Network Interfaces</strong></caption>\n");
	inittable();
	groups_enum(GROUPS_IF,listtable);
	printf("</table>\n");

	printf("</BODY></HTML>\n");
	exit(0);
}

