/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <net/if.h>
#include "../lib/defs.h"
#include "ifcfg.h"

char iface[16];
char parentiface[16];
int  childstatus;
int  childboot;
int  parentstatus;
int  parentboot;
int checkparentstatus(char *name,char *type,char *val)
{
	struct IFDATA ifd;
	if( strcmp(parentiface,name) )
		return(0);
	ifcfg_getifdata(name,&ifd);
	if(ifd.flags&IFF_UP)
		parentstatus=1;
	if(ifd.onboot)
		parentboot=1;
	return(0);
}
int checkchildstatus(char *name,char *type,char *val)
{
	struct IFDATA ifd;
	if( strncmp(iface,name,strlen(iface)) )
		return(0);
	if( !index(name,':') )
		return(0);
	ifcfg_getifdata(name,&ifd);
	if(ifd.flags&IFF_UP)
		childstatus=1;
	if(ifd.onboot)
		childboot=1;
	return(0);
}
int main(int argc,char *argv[])
{
	char str[256];
	char status[8];
	char boot_status[8];
	char tmp[5];
	int a0,a1,a2,a3,mask,tmps;
	unsigned int addr,host;
	struct IFDATA ifd;

	if(html_system(CHECK_LOGIN)){
		exit(10);
	}

	fgets(str,256,stdin);
	getparam(str,"iface=",iface,16);
	getparam(str,"addr0=",tmp,4);a0=atol(tmp);
	getparam(str,"addr1=",tmp,4);a1=atol(tmp);
	getparam(str,"addr2=",tmp,4);a2=atol(tmp);
	getparam(str,"addr3=",tmp,4);a3=atol(tmp);
	getparam(str,"mask=",tmp,4);mask=atol(tmp);
	getparam(str,"current_status=",status,8);
	getparam(str,"boot_status=",boot_status,8);

	/* check that the IP address is valid */
	if( a0==127 ){
		IFCFG_ERROR("error_3");exit(10);
	}
	if( a0==0 ){
		IFCFG_ERROR("error_4");exit(10);
	}
	if( a0>223 ){
		IFCFG_ERROR("error_5");exit(10);
	}
	if( (a1<0)||(a1>255)||(a2<0)||(a2>255)||(a3<0)||(a3>255) ){
		IFCFG_ERROR("error_12");exit(10);
	}

	/* check that the netmask is valid */
	if( (mask<8) ){
		IFCFG_ERROR("error_1");exit(10);
	}
	if( (mask>30) ){
		IFCFG_ERROR("error_2");exit(10);
	}

	/* check that host-address is valid */
	addr=(a0<<24)|(a1<<16)|(a2<<8)|a3;
	host=addr&((1<<(32-mask))-1);
	if( host==((1<<(32-mask))-1) ){
		IFCFG_ERROR("error_6");exit(10);
	}
	if( host==0 ){
		IFCFG_ERROR("error_7");exit(10);
	}

	/* all aliases follow parent */
	/* check the status of child interfaces */
	if( !index(iface,':') ){
		childstatus=0;
		childboot=0;
		groups_enum(GROUPS_IF,checkchildstatus);
		if( !strcmp(status,"down") ){
			/* if a primary interface is taken DOWN, all
			   childs must be down first */
			if(childstatus){
				IFCFG_ERROR("error_8");exit(10);
			}
		}
		if( !strcmp(boot_status,"down") ){
			/* if a primary interface is taken DOWN, all
			   childs must be down first */
			if(childboot){
				IFCFG_ERROR("error_9");exit(10);
			}
		}
	}
	/* check the status of parent interfaces */
	if( index(iface,':') ){
		strcpy(parentiface,iface);
		*index(parentiface,':')=0; /* name of parent iface */
		parentstatus=0;
		parentboot=0;
		groups_enum(GROUPS_IF,checkparentstatus);
		if( !strcmp(status,"up") ){
			/* if an alias interface is taken UP, the
			   parent must be UP also */
			if(!parentstatus){
				IFCFG_ERROR("error_10");exit(10);
			}
		}
		if( !strcmp(boot_status,"up") ){
			/* if an alias interface is taken UP, the
			   parent must be UP also */
			if(!parentboot){
				IFCFG_ERROR("error_11");exit(10);
			}
		}
	}

	/* everything seems OK, dodastuff */
	ifcfg_getifdata(iface,&ifd);
	if(!strcmp(status,"up")){
		ifd.flags|=IFF_UP;
	} else {
		ifd.flags&=(~IFF_UP);
	}
	if(!strcmp(boot_status,"up")){
		ifd.onboot=1;
	} else {
		ifd.onboot=0;
	}
	ifd.a0=a0;
	ifd.a1=a1;
	ifd.a2=a2;
	ifd.a3=a3;
	ifd.mask=mask;
	tmps=(~((1L<<(32-mask))-1));
	ifd.b0=(ifd.a0|((~tmps)>>24))&0xff;
	ifd.b1=(ifd.a1|((~tmps)>>16))&0xff;
	ifd.b2=(ifd.a2|((~tmps)>> 8))&0xff;
	ifd.b3=(ifd.a3|((~tmps)    ))&0xff;

	ifd.n0=ifd.a0&(tmps>>24)&0xff;
	ifd.n1=ifd.a1&(tmps>>16)&0xff;
	ifd.n2=ifd.a2&(tmps>> 8)&0xff;
	ifd.n3=ifd.a3&(tmps    )&0xff;

	ifcfg_setifdata(iface,&ifd);
	ifcfg_writeifdata(iface,&ifd);
	html_system(IFCFG_RELOADIFCFG);	
	return(0);
}

