/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

/* global defines for firewall project */
#ifndef TOPDIR 
#include <sys/types.h>

/* timeout used for autologout (seconds) */
#define LOGIN_TIMEOUT	300
#define LOGOUT		"/cgi-bin/fwt/login/logout"
#define CHECK_LOGIN	"/cgi-bin/fwt/login/checklogin"
#define RELOAD_RULES	"/cgi-bin/fwt/rules/reloadrules"

/* Apache directory */
/* path must be terminated by "/" */
#define TOPDIR 		"/home/httpd/"
/* directory containing all ifcfg-ethX files */
#define IFCFGDIR 	"/etc/sysconfig/network-scripts"
/* file containing among other, default route and if */
#define SYSNETWORK 	"/etc/sysconfig/network"
/* ppp config dir */
#define PPPDIR		"/etc/ppp"
/* ppp connection log */
#define PPPLOG		"/etc/ppp/ppplog"
/* static route file */
#define STATICROUTES 	"/etc/sysconfig/static-routes"
/* system log */
#define SYSTEM_LOG	"/var/log/messages"

/* firewall config files */
#define DB_GROUPS	"/etc/fwt/groups"
#define SP_GROUPS	"/etc/fwt/groups.special"
#define DB_FILTER	"/etc/fwt/filters"
#define FILTER_DIR	"/etc/fwt/flt"
#define DB_PORTS	"/etc/fwt/ports"
#define SP_PORTS	"/etc/fwt/ports.special"
#define WK_PORTS	"/etc/fwt/ports.wellknown"
#define RULES_INPUT	"/etc/fwt/rules.input"
#define RULES_OUTPUT	"/etc/fwt/rules.output"
#define RULES_FORWARD	"/etc/fwt/rules.forward"
#define TCPREDIR_FILE	"/etc/fwt/tcp-redirections"
#define TCPREDIR_TARGET	"/etc/fwt/tcp-redirection-targets"
#define ROUTECFG	"/etc/fwt/routes"
#define POLICYROUTERS	"/etc/fwt/policyrouters"
#define POLICYROUTING	"/etc/fwt/policyrouting"

/* mfw values */
#define MFW_REDIRECT_BASE	100
#define MFW_REDIRECT_NUM	 32
#define MFW_POLROUTE_BASE	(MFW_REDIRECT_BASE+MFW_REDIRECT_NUM)
#define MFW_POLROUTE_NUM	32

/* routing table priority offset for policy routes */
#define PR_PRIO_BASE		10000

/* system binaries */
#define IFCONFIG 	"/sbin/ifconfig"
#define IPCHAINS 	"/sbin/ipchains"
#define PPPD		"/usr/sbin/pppd"
#define IPMASQADM	"/usr/sbin/ipmasqadm"
#define ROUTE		"/sbin/route"
#define MODPROBE	"/sbin/modprobe"
#define IPTOOL		"/bin/ip"

/* rc script file */
#define FWT_RC_FILE	"/etc/rc.d/init.d/fwt"
/* forwarding control file */
#define IP_FORWARDING	"/proc/sys/net/ipv4/ip_forward"

/* main progs */
#define INPUT_RULES		"/cgi-bin/fwt/rules/rulesinput"
#define MAIN_COMMIT		"/cgi-bin/fwt/commit/commit"
#define FILTERS_MAIN		"/cgi-bin/fwt/filters/filters"
#define MAIN_GROUPS		"/cgi-bin/fwt/groups/groups"
#define PORTS_MAIN		"/cgi-bin/fwt/ports/ports"
#define IFCFG_MAIN		"/cgi-bin/fwt/ifconfig/ifconfig"
#define PPP_MAIN		"/cgi-bin/fwt/ppp/pppmain"
#define ROUTE_MAIN		"/cgi-bin/fwt/route/route"
#define POLROUTE_MAIN		"/cgi-bin/fwt/polroute/polroute"
#define LOG_MAIN		"/cgi-bin/fwt/log/log"

/* printlink lines */
#define FWT_LINKS \
"<A HREF=\""##INPUT_RULES##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/rules.gif\" ALT=\"[ Rules ]\"></A>\n \
 <A HREF=\""##MAIN_GROUPS##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/groups.gif\" ALT=\"[ Groups ]\"></A>\n \
 <A HREF=\""##PORTS_MAIN##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/ports.gif\" ALT=\"[ Ports ]\"></A>\n \
 <A HREF=\""##FILTERS_MAIN##"\" TARGET=\"_self\"><IMG src=\"/fwt/images/filters.gif\" ALT=\"[ Filters ]\"></A>\n \
 <A HREF=\""##MAIN_COMMIT##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/commitchanges.gif\" ALT=\"[ Commit changes ]\"></A>\n"
#define IFCFG_LINKS \
"<A HREF=\""##IFCFG_MAIN##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/ethernet.gif\" ALT=\"[ Ethernet ]\"></A>\n \
 <A HREF=\""##PPP_MAIN##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/ppp.gif\" ALT=\"[ PPP ]\"></A>\n \
 <A HREF=\""##ROUTE_MAIN##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/routing.gif\" ALT=\"[ Routing ]\"></A>\n \
 <A HREF=\""##POLROUTE_MAIN##"\" 	TARGET=\"_self\"><IMG src=\"/fwt/images/polrouting.gif\" ALT=\"[ Policy Routing ]\"></A>\n"

/* support binaries */
#define IFIP TOPDIR ## "/cgi-bin/fwt/ifconfig/ifip"
#define IFBC TOPDIR ## "/cgi-bin/fwt/ifconfig/ifbc"
#define IFNW TOPDIR ## "/cgi-bin/fwt/ifconfig/ifnw"
#define IFNM TOPDIR ## "/cgi-bin/fwt/ifconfig/ifnm"

/* function prototypes */
int html_system(char *path);
int html_title(char *title);
int getparam(char *params,char *name,char *val,int len);
void printlinks(char *line1,char *line2);
int option(char *selected,char *value,char *name);

#define PORTS_ALL	0
#define PORTS_DB	1
#define PORTS_WK	2
#define PORTS_SP	3
int ports_enum(int type,int (*ports_callback)(char *name,char *type,int port1,int port2));
int ports_num(int type);
int ports_getportsbyname(int type,char *name,int *port1,int *port2);
int ports_addport(char *name,int port1,int port2);
int ports_delport(char *name);

#define GROUPS_ALL		0
#define GROUPS_DB		1
#define GROUPS_SP		2
#define GROUPS_IF		3
#define GROUPS_IFNET		4
#define GROUPS_LLH		5
int groups_enum(int type,int (*groups_callback)(char *name,char *type,char *val));
int groups_num(int type);
char *groups_getgroupvalbyname(int type,char *name);
int groups_addgroup(char *name,char *val);
int groups_delgroup(char *name);
int groups_groupingroup(char *subgroup,char *group);

int icmp_typeenum(int (*callback)(char *name,char *value));
int icmp_codeenum(int (*callback)(char *name,char *value));


struct RULE {
	char action[20];
	char protocol[20];
	char iface[20];
	char srcip[20];
	char srcport[20];
	char dstip[20];
	char dstport[20];
	char icmptype[40];
	char icmpcode[40];
};
#define RULES_ALL	0
#define RULES_IN	1
#define RULES_OU	2
#define RULES_FW	3
int rules_enum(int type,int (*rules_callback)(struct RULE *rule));
int rules_num(int type);
struct RULE *rules_getrulebyindex(int type,int idx);
int rules_numgrouprefs(int type,char *group);
int rules_numportrefs(int type,char *port);
int rules_addrule(int type,int idx,struct RULE *rule);
int rules_delrule(int type,int idx);
int rules_httptorule(char *http,struct RULE *rule);
int rules_printashtml(struct RULE *rule);

struct REDIR {
	int mark;
	char iface[8];
	char port[20];
};
struct REDIR_TARGET {
	int mark;
	char host[24];
	char port[20];
};
int redir_enum(int (*callback)(struct REDIR *redir));
int redir_targetenum(int mark,int (*callback)(struct REDIR_TARGET *redir));
struct REDIR *redir_getredirbymark(int mark);
int redir_getmarkbyredir(char *iface,char *port);
int redir_create(char *iface,char *port);
int redir_addtarget(int mark,char *target,char *port);
int redir_deltarget(int mark,char *target,char *port);
int redir_del(int mark);

int filter_enum(int (*callback)(char *name,char *state,char *module,char *file,char *help));


struct IFDATA {
	short flags;
	int onboot;
	int a0,a1,a2,a3;
	int mask;
	int d0,d1,d2,d3; /* destination */
	int b0,b1,b2,b3; /* broadcast */
	int n0,n1,n2,n3; /* network */
};
int ifcfg_countbits(u_int32_t i);
char *ifcfg_masktostring(int mask);
int ifcfg_ifdown(char *iface);
int ifcfg_ifup(char *iface);
int ifcfg_setifdata(char *iface,struct IFDATA *ifdata);
int ifcfg_writeifdata(char *iface,struct IFDATA *ifdata);
int ifcfg_getifdata(char *iface,struct IFDATA *ifdata);

struct PPPDATA {
	char name[8];
	int enabled;
	int connected;
	int speed;
	char dev[8];
	char auth[8];
	char phone[40];
	char username[10];
};
int ppp_getpppdata(char *iface,struct PPPDATA *pdata);
int ppp_savepppdata(struct PPPDATA *pdata);
int ppp_savesecurity(char *iface,char *password);
int ppp_deleteppp(char *iface);
int ppp_checksecurity(char *iface);

struct STROUTE {
	char target[16];
	char ttype[8];	/* target type: "host" "net" or "default" */
	char iface[8];
	char gateway[16];
};
#define ROUTES_ALL	0
#define ROUTES_IFC	1
#define ROUTES_USR	2
int route_enum(int type,int (*route_callback)(char *typename,struct STROUTE *stroute));
int route_num(int type);
int route_numlocalroutestogroup(char *group);
struct STROUTE *route_getroutebytarget(char *target);
char *route_getgroupiface(char *group);
int route_addroute(char *target,char *gateway);
int route_delroute(char *target);
int route_updatesysfiles(void);


struct POLROUTER {
	int mark;
	char router[20];
};
struct POLROUTING {
	int mark;
	char protocol[20];
	char iface[20];
	char srcip[20];
	char srcport[20];
	char dstip[20];
	char dstport[20];
	char icmptype[40];
	char icmpcode[40];
};
int pr_enum(int (*callback)(struct POLROUTER *pr));
int pr_num(void);
struct POLROUTER *pr_getbymark(int mark);
struct POLROUTER *pr_getbyrouter(char *router);
int pr_add(char *router);
int pr_del(char *router);

int pr_enumroutes(int mark,int (*callback)(struct POLROUTING *pr));
int pr_numroutes(int mark);
int pr_addroute(int pos,struct POLROUTING *pr);
int pr_delroute(int pos,int mark);
struct POLROUTING *pr_getroutebypos(int pos,int mark);

#endif


