/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

/* groups_enum.c
** int groups_enum(int type,int (*callback)(char *name,char *type,char *val)
** this function will iterate over all specified groups and call 
** callback for each of them (if callback is non NULL)
**			
** type is either
** GROUPS_ALL   : All groups known to the firewall
** GROUPS_DB    : all user defined groups
** GROUPS_SP    : all special groups
** GROUPS_IF    : all interface groups
** GROUPS_IFNET : all interfaces plus network and broadcast address
** GROUPS_LLH   : all groups that are single hosts on a local link
**
** function will return the number of groups iterated over
*/

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <dirent.h>
#include <linux/unistd.h>
#include "../defs.h"

static int groups_enum_file(int tp,char *filename,char *type,int (*callback)
		(char *name,char *type,char *val))
{
	int num;
	FILE *fh;
	char name[20],val[30];

	num=0;
	if(fh=fopen(filename,"r")){
		while(!feof(fh)){
			if(fscanf(fh,"%s %s",name,val)==2){
				/* special check for LLH, only count single
				   hosts on local links here */
				if(tp==GROUPS_LLH){
					char ts[16];
					/* only interested in hosts */
					if(!strstr(val,"/32")){
						continue;
					}
					strcpy(ts,name);
					if(!route_numlocalroutestogroup(ts)){
						continue;
					}
				}
				if(callback){
					(*callback)(name,type,val);
				}
				num++;	
			}
		}
		fclose(fh);
	}
	return(num);
}

/* iftype :
** 0: only the if is iterated
** 1: the if as well as net and bcst for lo and eth is iterated
*/
static int groups_enum_if(int iftype,int (*callback)
		(char *name,char *type,char *val))
{
	int num;
	DIR *dir;
	struct dirent *dirent;
	static char val[30];

	num=0;
	if(dir=opendir(IFCFGDIR)){
	    while(dirent=readdir(dir)){
		if(!strncmp("ifcfg-",dirent->d_name,6)){
		   if(callback){
			sprintf(val,"$%s",&dirent->d_name[6]);
			(*callback)(val+1,"interface",val);
		   }
		   num++;
		   if( (iftype==1) 
		   && (  !strncmp(&dirent->d_name[6],"eth0",3)
		      || !strncmp(&dirent->d_name[6],"lo",2))){
			if(callback){
			   sprintf(val,"$%s-net",&dirent->d_name[6]);
			   (*callback)(val+1,"network",val);
			   sprintf(val,"$%s-bcs",&dirent->d_name[6]);
			   (*callback)(val+1,"broadcast",val);
			}
			num+=2;
		   }
		}
	    }
	    closedir(dir);
	}
	return(num);
}	
	
int groups_enum(int type,int (*callback)(char *name,char *type,char *val))
{
	int num;

	num=0;
	if((type==GROUPS_ALL)||(type==GROUPS_SP)){
		num+=groups_enum_file(type,SP_GROUPS,"special",callback);
	}
	if((type==GROUPS_ALL)||(type==GROUPS_DB)||(type==GROUPS_LLH)){
		num+=groups_enum_file(type,DB_GROUPS,"user defined",callback);
	}
	if(type==GROUPS_ALL){
		num+=groups_enum_if(1,callback);
	} else {
		if(type==GROUPS_IF){
			num+=groups_enum_if(0,callback);
		}
		if(type==GROUPS_IFNET){
			num+=groups_enum_if(1,callback);
		}
	}

	return(num);
}

