/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */
/* groups_groupingroup.c
 * this function checks if one group is a subgroup of another, i.e. is
 * contained completely inside another group.
 * i.e. 1.1.1.1/32 would be a subgroup of 1.0.0.0/8 but
 * 1.1.1.0/24 would not be a subgroup of 1.0.0.0/16
 *
 * function will return :
 * 1: group is a subgroup
 * 0: group is not a subgroup (or error)
 */
/*XXXXX this function is very much a hack and should be completely
        rewritten.
  It assumes all groups starting with '$' as value to be interfaces
  and treats by calculating that interface' network address and mask.
  Will break completely if handed a $eth0_net or similar.
*/
#include <stdio.h>
#include "../defs.h"


int groups_groupingroup(char *subgroup,char *group)
{
	char *str;
	unsigned int a0,a1,a2,a3;
	unsigned int grmask,sgrmask;
	unsigned long gr,sgr;
	struct IFDATA ifdata;

	if( (str=groups_getgroupvalbyname(GROUPS_ALL,group))==NULL ){
		return(0);
	}
	if(str[0]=='$'){	/* group was an interface */
		ifcfg_getifdata(str+1,&ifdata);
		a0=ifdata.a0;		
		a1=ifdata.a1;		
		a2=ifdata.a2;		
		a3=ifdata.a3;
		grmask=ifdata.mask;		
	} else {
		sscanf(str,"%d.%d.%d.%d/%d",&a0,&a1,&a2,&a3,&grmask);
	}
	gr =a0;gr<<=8;
	gr|=a1;gr<<=8;
	gr|=a2;gr<<=8;
	gr|=a3;

	if( (str=groups_getgroupvalbyname(GROUPS_ALL,subgroup))==NULL ){
		return(0);
	}
	sscanf(str,"%d.%d.%d.%d/%d",&a0,&a1,&a2,&a3,&sgrmask);
	sgr =a0;sgr<<=8;
	sgr|=a1;sgr<<=8;
	sgr|=a2;sgr<<=8;
	sgr|=a3;

	if(sgrmask<grmask){	/*subgroup can not be bigger than the group */
		return(0);
	}
	if(grmask>30){		/* the group must be bigger than a single */
				/* to host contain a subgroup */
		return(0);
	}

	gr&=(~((1UL<<(32-grmask))-1));
	sgr&=(~((1UL<<(32-grmask))-1));
	if( gr^sgr ){
		return(0);
	}

	return(1);
}

