/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

/*ifcfg_getifdata.c
**
*/

#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include "../defs.h"

int ifcfg_getifdata(char *ifname,struct IFDATA *ifd)
{
	int sfd,res;
	FILE *fd;
	struct ifreq ifr;
	char tmps[100];
	int a0,a1,a2,a3;
	unsigned int mask;

	/* read info from config file */
	ifd->onboot=0;
	sprintf(tmps,"%s/ifcfg-%s",IFCFGDIR,ifname);
	fd=fopen(tmps,"r");
	while(fgets(tmps,99,fd)){
		if(!strncmp(tmps,"ONBOOT=",7)){
			switch(tmps[7]){
			case 'y':
			case 'Y':
				ifd->onboot=1;
			}
		}
		if(!strncmp(tmps,"IPADDR=",7)){
			sscanf(tmps+7,"%d.%d.%d.%d\n",
				&ifd->a0,&ifd->a1,&ifd->a2,&ifd->a3);
		}
		if(!strncmp(tmps,"BROADCAST=",10)){
			sscanf(tmps+10,"%d.%d.%d.%d\n",
				&ifd->b0,&ifd->b1,&ifd->b2,&ifd->b3);
		}
		if(!strncmp(tmps,"NETWORK=",8)){
			sscanf(tmps+8,"%d.%d.%d.%d\n",
				&ifd->n0,&ifd->n1,&ifd->n2,&ifd->n3);
		}
		if(!strncmp(tmps,"NETMASK=",8)){
			sscanf(tmps+8,"%d.%d.%d.%d\n",&a0,&a1,&a2,&a3);
			ifd->mask=ifcfg_countbits(a0)
				+ifcfg_countbits(a1)
				+ifcfg_countbits(a2)
				+ifcfg_countbits(a3);
		}
				
	}
	fclose(fd);

	/* check up/down status using ioctl */
	sfd=socket(AF_INET,SOCK_DGRAM,0);
	memcpy(ifr.ifr_name,ifname,IFNAMSIZ);

	/* flags */
	ifd->flags=0;
	if(res=ioctl(sfd,SIOCGIFFLAGS,&ifr))
		return(res);
	ifd->flags=ifr.ifr_ifru.ifru_flags;

	/* if interface is up, we reread the address (to keep ppp happy)*/
	if( ifd->flags&IFF_UP ){
		if(res=ioctl(sfd,SIOCGIFADDR,&ifr))
			return(res);
		ifd->a0=ifr.ifr_ifru.ifru_addr.sa_data[2]&0xff;
		ifd->a1=ifr.ifr_ifru.ifru_addr.sa_data[3]&0xff;
		ifd->a2=ifr.ifr_ifru.ifru_addr.sa_data[4]&0xff;
		ifd->a3=ifr.ifr_ifru.ifru_addr.sa_data[5]&0xff;
	}

	/* destination */
	if(res=ioctl(sfd,SIOCGIFDSTADDR,&ifr))
		return(res);
	ifd->d0=ifr.ifr_ifru.ifru_addr.sa_data[2]&0xff;
	ifd->d1=ifr.ifr_ifru.ifru_addr.sa_data[3]&0xff;
	ifd->d2=ifr.ifr_ifru.ifru_addr.sa_data[4]&0xff;
	ifd->d3=ifr.ifr_ifru.ifru_addr.sa_data[5]&0xff;
	close(sfd);

	return(res);
}


