/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

/* ports_enum.c
** port enumerator
** int port_enum(int type,int (*port_callback)(char *name,int port1,int port2))
** this function will iterate over all defined ports of the specified type
** and if port_callback
** is non-NULL it will be called once for each port defined.
** type can be 
** PORTS_ALL : all ports known to firewall
** PORTS_DB  : all user defined ports in file DB_PORTS
** PORTS_SP  : all special ports statically defined in SP_PORTS
** PORTS_WK  : all wellknown ports as defined in WK_PORTS
**
** function will return number of ports it iterated over.
**
** callback parameters :
** name : name of port (range)
** type : string describing type of port, e.g. "Well Known"
** port1: first port in range, or single port if port2==0
** port2: if>0 end of portrange
*/
#include <stdio.h>
#include <string.h>
#include "../defs.h"

static int ports_enum_file(char *filename,char *type,int (*port_callback)(char *name,char *type,int port1,int port2));


int ports_enum(int type,int (*port_callback)(char *name,char *type,int port1,int port2))
{
	int num;
	num=0;
	if((type==PORTS_ALL)||(type==PORTS_SP)){
		num+=ports_enum_file(SP_PORTS,"special",port_callback);
	}
	if((type==PORTS_ALL)||(type==PORTS_WK)){
		num+=ports_enum_file(WK_PORTS,"well known",port_callback);
	}
	if((type==PORTS_ALL)||(type==PORTS_DB)){
		num+=ports_enum_file(DB_PORTS,"user defined",port_callback);
	}
	return(num);
}

static int ports_enum_file(char *filename,char *type,int (*port_callback)(char *name,char *type,int port1,int port2))
{
	FILE *fh;
	char name[20];
	int port1,port2,num;
	num=0;
	if(fh=fopen(filename,"r")){
		while(!feof(fh)){
			port2=0;
			if(fscanf(fh,"%s %d:%d",name,&port1,&port2)>1){
				if(port_callback){
					(*port_callback)(name,type,port1,port2);
				}
				num++;
			}
		}
		fclose(fh);
	}
	return(num);
}

