/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */
/* route_addroute.c
 * This function will create a new route.
 * Before adding the new route, it will first try to delete any
 * already existing routes to the same target.
 * The default route needs some special handling since it is kept
 * in a different config file on standard RedHat systems.
 * The function will also try to add the new route at a position so
 * that the order will be from most specific route to least specific route
 * (i.e. the same order as they really are handled by the kernel)
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "../defs.h"

static int flag;
static FILE *fh;
static struct STROUTE stroute;

static int callback(char *typename,struct STROUTE *str)
{
	int a0,a1,a2,a3,n1,n2;
	/* overly complex, but we want to try to keep the order the
           same as the kernel will use internally so it looks
           nice on the table on the main route page */
	if(!flag){
		if(!strcmp("default",stroute.target)){
			n1=0;
		} else {
			sscanf(groups_getgroupvalbyname(GROUPS_DB,stroute.target),
				"%d.%d.%d.%d/%d\n",&a0,&a1,&a2,&a3,&n1);
		}
		if(!strcmp(str->target,"default")){
			n2=0;
		} else {
			sscanf(groups_getgroupvalbyname(GROUPS_DB,str->target),
				"%d.%d.%d.%d/%d\n",&a0,&a1,&a2,&a3,&n2);
		}
		if(n1>=n2){
			fprintf(fh,"%s %s %s %s\n",
				stroute.iface,stroute.target,
				stroute.gateway,stroute.ttype);
			flag=1;
		}
	}
	fprintf(fh,"%s %s %s %s\n",
		str->iface,str->target,
		str->gateway,str->ttype);
	return(0);
}

int route_addroute(char *target,char *gateway)
{
	int a0,a1,a2,a3,nm;
	char cmd[256],str[32];
	char tg[32];

	strcpy(stroute.target,target);
	strcpy(stroute.gateway,gateway);
	strcpy(stroute.iface,route_getgroupiface(gateway)); /*already checked*/
	strcpy(str,groups_getgroupvalbyname(GROUPS_DB,gateway));
	if(index(str,'/'))
		*index(str,'/')=0;
	if(!strcmp("default",stroute.target)){
		strcpy(stroute.ttype,"default");
		sprintf(cmd,"%s del -net 0.0.0.0 netmask 0.0.0.0 >/dev/null 2>/dev/null",ROUTE);
		system(cmd);
		sprintf(cmd,"%s add -net 0.0.0.0 gw %s netmask 0.0.0.0 >/dev/null 2>/dev/null",
			ROUTE,
			str
		);
		system(cmd);
	} else {
		sscanf(groups_getgroupvalbyname(GROUPS_DB,target),
			"%d.%d.%d.%d/%d\n",&a0,&a1,&a2,&a3,&nm);
		if(nm==32){
			strcpy(tg,groups_getgroupvalbyname(GROUPS_DB,target));
			if(index(tg,'/'))*index(tg,'/')=0;
			strcpy(stroute.ttype,"host");
/*#ifdef REMOVED*/
			sprintf(cmd,"%s del -host %s >/dev/null 2>/dev/null",
				ROUTE,
				tg
			);
			system(cmd);
/*#endif*/
			sprintf(cmd,"%s add -host %s gw %s >/dev/null 2>/dev/null",
				ROUTE,
				tg,
				str
			);
			system(cmd);
		} else {
			strcpy(tg,groups_getgroupvalbyname(GROUPS_DB,target));
			if(index(tg,'/'))*index(tg,'/')=0;
			strcpy(stroute.ttype,"net");
/*#ifdef REMOVED*/
			sprintf(cmd,"%s del -net %s netmask %s >/dev/null 2>/dev/null",
				ROUTE,
				tg,
				ifcfg_masktostring(nm)
			);
			system(cmd);
/*#endif*/
			sprintf(cmd,"%s add -net %s gw %s netmask %s >/dev/null 2>/dev/null",
				ROUTE,
				tg,
				str,
				ifcfg_masktostring(nm)
			);
			system(cmd);
		}
	}

	flag=0;
	fh=fopen(ROUTECFG".new","w");
	route_enum(ROUTES_USR,callback);	
	if(!flag){
		fprintf(fh,"%s %s %s %s\n",
			stroute.iface,stroute.target,
			stroute.gateway,stroute.ttype);
	}
	fclose(fh);

	unlink(ROUTECFG);
	rename(ROUTECFG".new",ROUTECFG);
	sprintf(cmd,"/bin/chown fwt:nobody %s >/dev/null 2>/dev/null",ROUTECFG);
	system(cmd);
	chmod(ROUTECFG,S_IRUSR|S_IWUSR);
	route_updatesysfiles();
	return(0);
}

