/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */
/*routes_enum.c
 * enumerator will call route_callback (if non-NULL) once for each of the
 * known routes of type:
 * ROUTES_ALL: all known routes
 * ROUTES_IFC: all automatically generated routes for ethernet interfaces
 *             (i.e. the route created for the link of an iface)
 * ROUTES_USR: all user defined routes
 *
 * routes_enum() will return the number of routes for the specified type
 */

#include <stdio.h>
#include <string.h>
#include <net/if.h>
#include "../defs.h"

static int matches;
/* these function just handles the automatic routes */
static int (*callback)(char *type,struct STROUTE *stroute);
static int route_automatic_callback(char *name,char *type,char *val)
{
	struct STROUTE stroute;
	struct IFDATA ifdata;
	/* only automatic routes on ethX which are UP*/
	if(!strncmp("eth",name,3)){
		ifcfg_getifdata(name,&ifdata);
		if(ifdata.flags&IFF_UP){
			matches++;
			if(callback){
				strcpy(stroute.target,name);
				strcat(stroute.target,"_net");
				strcpy(stroute.ttype,"net");
				strcpy(stroute.iface,name);
				strcpy(stroute.gateway,"");
				callback("interface",&stroute);
			}
		}
	}
}

/* this function just handles the user defined routes */
static int route_enum_userdefined()
{
	FILE *fh;
	char str[256];
	struct STROUTE stroute;
	if(fh=fopen(ROUTECFG,"r")){
		while(!feof(fh)){
			str[0]=0;
			fgets(str,255,fh);
			if(sscanf(str,"%s %s %s %s\n",
				stroute.iface,stroute.target,
				stroute.gateway,stroute.ttype)==4){
				matches++;
				if(callback)
					(*callback)("user defined",&stroute);
			}
		}
		fclose(fh);
	}
	return(0);
}

int route_enum(int type,int (*route_callback)(char *typename,struct STROUTE *stroute))
{	
	matches=0;
	callback=route_callback;
	if( (type==ROUTES_ALL)||(type==ROUTES_IFC) ){
		groups_enum(GROUPS_IF,route_automatic_callback);
	}
	if( (type==ROUTES_ALL)||(type==ROUTES_USR) ){
		route_enum_userdefined();
	}
	return(matches);
}

