/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

/*rules_addrule.c
** int rules_addrule(int type,int idx,struct RULE *rule)
** this function will insert the rule rule at index idx in the
** rulelist specified by type.
** if idx is outside the range, the rule will be added to the end.
**
** type can be
** RULES_IN
** RULES_OU
** RULES_FW
*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "../defs.h"

static int xflag;
static FILE *xfh;
static int xidx;
static struct RULE *xrule;
static int static_callback(struct RULE *rule)
{
	if(!xidx--){
		fprintf(xfh,"%s %s %s %s %s %s %s %s %s\n",
			xrule->action,xrule->protocol,xrule->iface,
			xrule->srcip,xrule->srcport,
			xrule->dstip,xrule->dstport,
			xrule->icmptype,xrule->icmpcode);
		xflag=1;
	}
	fprintf(xfh,"%s %s %s %s %s %s %s %s %s\n",
		rule->action,rule->protocol,rule->iface,
		rule->srcip,rule->srcport,
		rule->dstip,rule->dstport,
		rule->icmptype,rule->icmpcode);
	return(0);
}

int rules_addrule(int type,int idx,struct RULE *rule)
{
	xflag=0;
	xrule=rule;
	xidx=idx;
	switch(type){
	case RULES_IN:
		xfh=fopen(RULES_INPUT".new","w");
		break;
	case RULES_OU:
		xfh=fopen(RULES_OUTPUT".new","w");
		break;
	case RULES_FW:
		xfh=fopen(RULES_FORWARD".new","w");
		break;
	default:
		exit(10);
	}
	rules_enum(type,static_callback);
	if(!xflag){
		fprintf(xfh,"%s %s %s %s %s %s %s %s %s\n",
			xrule->action,xrule->protocol,xrule->iface,
			xrule->srcip,xrule->srcport,
			xrule->dstip,xrule->dstport,
			xrule->icmptype,xrule->icmpcode);
	}
	fclose(xfh);
	switch(type){
	case RULES_IN:
		unlink(RULES_INPUT);
		rename(RULES_INPUT".new",RULES_INPUT);
		chmod(RULES_INPUT,S_IRUSR|S_IWUSR);
		break;
	case RULES_OU:
		unlink(RULES_OUTPUT);
		rename(RULES_OUTPUT".new",RULES_OUTPUT);
		chmod(RULES_OUTPUT,S_IRUSR|S_IWUSR);
		break;
	case RULES_FW:
		unlink(RULES_FORWARD);
		rename(RULES_FORWARD".new",RULES_FORWARD);
		chmod(RULES_FORWARD,S_IRUSR|S_IWUSR);
		break;
	}
	return(0);
}






