/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

/*rules_enum.c
**int rules_enum(int type,int (*callback)(struct RULE *rule))
**function will iterate over all rules of matching type and return the
**number of iterations.
**if callback is non NULL it will be called once for each matching rule.
**
** type can be
** RULES_ALL : iterate over all known rules
** RULES_IN  : iterate only over the set of rules on INPUT chain
** RULES_OU  : iterate only over the set of rules on OUTPUT chain
** RULES_FW  : iterate only over the set of rules on FORWARD chain
**
** function will return 
**  number of matching rules
*/
#include <stdio.h>
#include "../defs.h"

static int rules_enum_file(char *filename,int (*callback)(struct RULE *rule))
{
	int num;
	FILE *fh;
	static struct RULE rule;

	num=0;
	if(fh=fopen(filename,"r")){
	    while(!feof(fh)){
		if(fscanf(fh,"%s %s %s %s %s %s %s %s %s",
			rule.action,rule.protocol,rule.iface,
			rule.srcip,rule.srcport,
			rule.dstip,rule.dstport,
			rule.icmptype,rule.icmpcode)==9){
		    if(callback){
			(*callback)(&rule);
		    }
		    num++;
		}
	    }
	    fclose(fh);
	}	
	return(num);
}

int rules_enum(int type,int (*callback)(struct RULE *rule))
{
	int num;

	num=0;
	if((type==RULES_ALL)||(type==RULES_IN)){
		num+=rules_enum_file(RULES_INPUT,callback);
	}
	if((type==RULES_ALL)||(type==RULES_OU)){
		num+=rules_enum_file(RULES_OUTPUT,callback);
	}
	if((type==RULES_ALL)||(type==RULES_FW)){
		num+=rules_enum_file(RULES_FORWARD,callback);
	}
	return(num);
}


