/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

#include <stdio.h>
#include <string.h>
#include "../lib/defs.h"
#include "polroute.h"

int routeid;

int inittable(void)
{
	printf("<table border>\n");
	printf("<caption><strong>Policy Routing Table</strong></caption>\n");
	printf("<tr>\n");
	printf("<th>Pos</th>\n");
	printf("<th>Protocol</th>\n");
	printf("<th>Inp Iface</th>\n");
	printf("<th>Src IP</th>\n");
	printf("<th>Src Port</th>\n");
	printf("<th>Dst IP</th>\n");
	printf("<th>Dst Port</th>\n");
	printf("<th>ICMP Type</th>\n");
	printf("<th>ICMP Code</th>\n");
	printf("<th></th>\n");
	printf("<th>Via Router</th>\n");
	printf("</tr>\n");
	return(0);
}
int tableroutes(struct POLROUTING *pr)
{
	printf("<tr>\n");
	printf("<td>%d</td>\n",routeid);
	printf("<td>%s</td>\n",pr->protocol);
	printf("<td>%s</td>\n",pr->iface);
	printf("<td>%s</td>\n",pr->srcip);
	printf("<td>%s</td>\n",pr->srcport);
	printf("<td>%s</td>\n",pr->dstip);
	printf("<td>%s</td>\n",pr->dstport);
	printf("<td>%s</td>\n",pr->icmptype);
	printf("<td>%s</td>\n",pr->icmpcode);
	printf("<td> => </td>\n");
	printf("<td>%s</td>\n",
		pr_getbymark(pr->mark)->router
	);
	printf("</tr>\n");
	routeid++;
	return(0);
}

int endtable(void)
{
	printf("</table>\n");
	return(0);
}

int listroutes(struct POLROUTING *pr)
{
	printf("<option value=%d>Pos:%d => %s\n",
		routeid,
		routeid,
		pr_getbymark(pr->mark)->router
	);
	routeid++;
	return(0);
}
int listaddrouter(char *name,char *type,char *val)
{
	printf("<option value=\"%s\">%s\n",name,name);
	return(0);
}
int polroutelist(struct POLROUTER *pr)
{
	printf("<option value=\"%s\">%s\n",pr->router,pr->router);
	return(0);
}
int listif(char *iface,char *type,char *val)
{
	printf("<option value=\"%s\">%s\n",iface,iface);
	return(0);
}
int listport(char *name,char *type,int port1,int port2)
{
	printf("<option value=\"%s\">%s\n",name,name);
	return(0);
}
int listicmp(char *name,char *val)
{
	printf("<option value=\"%s\">%s\n",val,name);
	return(0);
}

int main(int argc,char *argv[])
{
	int i,j;

	if(html_system(CHECK_LOGIN)){
		exit(10);
	}

	html_title("Policy Routing");
	printf("<BODY>\n");
	printlinks(IFCFG_LINKS,NULL);
	printf("<H3>Policy Routing</H3>\n");

	/* add router as policy router */
	if(groups_num(GROUPS_LLH)){
		printf("<form action=\"%s\" method=post>\n",QUERYADDROUTER_POLROUTE);
		printf("<strong>Router:</strong>\n");
		printf("<select name=router>\n");
		groups_enum(GROUPS_LLH,listaddrouter);
		printf("</select>\n");
		printf("<input type=submit value=\"Add Policy Router\">\n");
		printf("<br>\n");
		printf("</form>\n");
	} else {
		printf("<strong>NO POSSIBLE ROUTERS DEFINED</strong><br>\n");
		printf("a router must be a <strong>group</strong> which has<br>\n");
		printf("been defined as a single host connected to a local<br>\n");
		printf("link.<br>\n");
	}

	/* delete policy router */
	if(pr_num()){
		printf("<form action=\"%s\" method=post>\n",QUERYDELROUTER_POLROUTE);
		printf("<strong>Router:</strong>\n");
		printf("<select name=router>\n");
		pr_enum(polroutelist);
		printf("</select>\n");
		printf("<input type=submit value=\"Delete Policy Router\">\n");
		printf("<br>\n");
		printf("</form>\n");
	}

	/* add new policy route (if we have any policy routers that is */
	if(pr_num()){
		printf("<form action=\"%s\" method=post>\n",QUERYADDPOLROUTE_POLROUTE);

		/* position */
		printf("<STRONG>Pos:</STRONG>\n");
		if((j=pr_numroutes(-1))>0){
			printf("<SELECT NAME=\"pos\">\n");
			for(i=0;i<j;i++){
				printf("<OPTION VALUE=\"%d\">%d\n",i,i);
			}
			printf("<OPTION VALUE=\"%d\">last\n",i);
			printf("</SELECT>\n");
		} else {
			printf("<STRONG>0</STRONG>\n");
			printf("<INPUT TYPE=\"hidden\" NAME=\"pos\" VALUE=\"0\">\n");
		}

		/* iface */
		printf("<STRONG>Iface:</STRONG>\n");
		printf("<SELECT NAME=\"iface\">\n");
		listif("ALL",NULL,NULL);
		groups_enum(GROUPS_IF,listif);
		printf("</SELECT>\n");

		/* protocol */
		printf("<STRONG>Protocol:</STRONG>\n");
		printf("<SELECT NAME=\"protocol\">\n");
		option(NULL,"ALL","ALL");
		option(NULL,"tcp","Tcp");
		option(NULL,"tcpsyn","Tcp + Syn");
		option(NULL,"udp","Udp");
		option(NULL,"icmp","Icmp");
		option(NULL,"frag","IP Fragment");
		printf("</SELECT>\n");
		printf("<BR>\n");

		/* SRC IP address */
		printf("<STRONG>SRC-IP:</STRONG>\n");
		printf("<SELECT NAME=\"srcip\">\n");
		groups_enum(GROUPS_ALL,listif);
		printf("</SELECT>\n");

		/* SRC port */
		printf("<STRONG>Ports:</STRONG>\n");
		printf("<SELECT NAME=\"srcport\">\n");
		ports_enum(PORTS_ALL,listport);
		printf("</SELECT>\n");

		/* ICMP Types */
		printf("<STRONG>ICMP Type:</STRONG>\n");
		printf("<SELECT NAME=\"icmptype\">\n");
		icmp_typeenum(listicmp);
		printf("</SELECT>\n");
		printf("<BR>\n");

		/* DST IP address */
		printf("<STRONG>DST-IP:</STRONG>\n");
		printf("<SELECT NAME=\"dstip\">\n");
		groups_enum(GROUPS_ALL,listif);
		printf("</SELECT>\n");

		/* DST port */
		printf("<STRONG>Ports:</STRONG>\n");
		printf("<SELECT NAME=\"dstport\">\n");
		ports_enum(PORTS_ALL,listport);
		printf("</SELECT>\n");

		/* ICMP Codes */
		printf("<STRONG>ICMP Code:</STRONG>\n");
		printf("<SELECT NAME=\"icmpcode\">\n");
		icmp_codeenum(listicmp);
		printf("</SELECT>\n");
		printf("<BR>\n");

		/* route by this plicy router */
		printf("<strong>Use This Policy Router:</strong>\n");
		printf("<select name=router>\n");
		pr_enum(polroutelist);
		printf("</select>\n");
		printf("<input type=submit value=\"Add This Policy Route\">\n");
		printf("<br>\n");
		printf("</form>\n");
	}

	/* delete policy route */
	if(pr_numroutes(-1)){
		printf("<form action=\"%s\" method=post>\n",QUERYDELPOLROUTE_POLROUTE);
		printf("<select name=\"pos\">\n");
		routeid=0;
		pr_enumroutes(-1,listroutes);
		printf("</select>\n");
		printf("<input type=submit value=\"Delete This Policy Route\">\n");
		printf("</form>\n");
	}

	inittable();
	routeid=0;
	pr_enumroutes(-1,tableroutes);	
	endtable();	

	printf("</BODY></HTML>\n");
	exit(0);
}
