/*
 *		This program is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 * Author:	Ronnie Sahlberg <ronnie.sahlberg@mbox301.swipnet.se>
 * Changes:
 */

#include <stdio.h>
#include <string.h>
#include "../lib/defs.h"
#include "rules.h"

char ruleset[16];
int pos;

int ruletable(struct RULE *rule)
{
	printf("<TR><TD><STRONG>%d</STRONG></TD>",pos);
	printf("<TD>%s</TD><TD>%s</TD><TD>%s</TD>",rule->action,rule->protocol,rule->iface);	
	printf("<TD>%s</TD>",rule->srcip);
	if( !strncmp(rule->protocol,"tcp",3) || !strcmp(rule->protocol,"udp") ){
		printf("<TD>%s</TD>",rule->srcport);
	} else {
		printf("<TD></TD>");
	}
	printf("<TD>%s</TD>",rule->dstip);
	if( !strncmp(rule->protocol,"tcp",3) || !strcmp(rule->protocol,"udp") ){
		printf("<TD>%s</TD>",rule->dstport);
	} else {
		printf("<TD></TD>");
	}
	if( !strcmp(rule->protocol,"icmp") ){
		printf("<TD>%s</TD><TD>%s</TD>",rule->icmptype,rule->icmpcode);
	} else {
		printf("<TD></TD><TD></TD>");
	}
	printf("</TR>\n");
	pos++;
	return(0);
}

int listrule(struct RULE *rule)
{
	printf("<OPTION VALUE=\"%d\">%d: %s %s %s %s %s %s %s\n",pos,pos,rule->action,rule->protocol,rule->iface,rule->srcip,rule->srcport,rule->dstip,rule->dstport);
	pos++;
	return(0);
}

int listif(char *iface,char *type,char *value)
{
	printf("<OPTION VALUE=\"%s\">%s\n",iface,iface);
	return(0);
}

int listport(char *name,char *type,int port1,int port2)
{
	printf("<OPTION VALUE=\"%s\">%s\n",name,name);
	return(0);
}
int listicmp(char *name,char *value)
{
	printf("<OPTION VALUE=\"%s\">%s\n",value,name);
	return(0);
}

int main(int argc,char *argv[])
{
	int i,j,type;
	char str[256];
	char *name,*Name;

	if(html_system(CHECK_LOGIN)){
		exit(10);
	}


	fgets(str,256,stdin);
	if(getparam(str,"ruleset=",ruleset,16)){
		strcpy(ruleset,"input");
	}
	if(!strcmp(ruleset,"input")){
		type=RULES_IN;Name="Input";name="input";
	} else if(!strcmp(ruleset,"output")){	
		type=RULES_OU;Name="Output";name="output";
	} else {
		type=RULES_FW;Name="Forward";name="forward";
	}
	
	sprintf(str,"FWT %s Rules",Name);
	html_title(str);
	printf("<BODY>\n");
	printlinks(FWT_LINKS,RULE_LINKS);
	printf("<H3>%s Rules</H3>\n",Name);
	printf("<FORM ACTION=\"%s\" METHOD=POST>\n",QUERY_ADD_RULE);
	/* position */
	printf("<STRONG>Pos:</STRONG>\n");
	if((j=rules_num(type))>0){
		printf("<SELECT NAME=\"selected_rule\">\n");
		for(i=0;i<j;i++){
			printf("<OPTION VALUE=\"%d\">%d\n",i,i);
		}
		printf("<OPTION VALUE=\"%d\">last\n",i);
		printf("</SELECT>\n");
	} else {
		printf("<STRONG>0</STRONG>\n");
		printf("<INPUT TYPE=\"hidden\" NAME=\"selected_rule\" VALUE=\"0\">\n");
	}

	/* iface */
	printf("<STRONG>Iface:</STRONG>\n");
	printf("<SELECT NAME=\"iface\">\n");
	listif("ALL",NULL,NULL);
	groups_enum(GROUPS_IF,listif);
	printf("</SELECT>\n");

	/* protocol */
	printf("<STRONG>Protocol:</STRONG>\n");
	printf("<SELECT NAME=\"protocol\">\n");
	option(NULL,"ALL","ALL");
	option(NULL,"tcp","Tcp");
	option(NULL,"tcpsyn","Tcp + Syn");
	option(NULL,"udp","Udp");
	option(NULL,"icmp","Icmp");
	option(NULL,"frag","IP Fragment");
	printf("</SELECT>\n");

	/* action */
	printf("<STRONG>Action:</STRONG>\n");
	printf("<SELECT NAME=\"action\">\n");
	printf("<OPTION VALUE=\"ACCEPT\">Accept\n");
	if(!strcmp(ruleset,"forward")){
		printf("<OPTION VALUE=\"MASQ\">Masquerade\n");
	}
	printf("<OPTION VALUE=\"REJECT\">Reject\n");
	printf("<OPTION VALUE=\"REJECTLOG\">Reject+Log\n");
	printf("<OPTION VALUE=\"DENY\">Deny\n");
	printf("<OPTION VALUE=\"DENYLOG\">Deny+Log\n");
	printf("</SELECT>\n");
	printf("<BR>\n");

	/* SRC IP address */
	printf("<STRONG>SRC-IP:</STRONG>\n");
	printf("<SELECT NAME=\"srcip\">\n");
	groups_enum(GROUPS_ALL,listif);
	printf("</SELECT>\n");

	/* SRC port */
	printf("<STRONG>Ports:</STRONG>\n");
	printf("<SELECT NAME=\"srcport\">\n");
	ports_enum(PORTS_ALL,listport);
	printf("</SELECT>\n");

	/* ICMP Types */
	printf("<STRONG>ICMP Type:</STRONG>\n");
	printf("<SELECT NAME=\"icmptype\">\n");
	icmp_typeenum(listicmp);
	printf("</SELECT>\n");
	printf("<BR>\n");

	/* DST IP address */
	printf("<STRONG>DST-IP:</STRONG>\n");
	printf("<SELECT NAME=\"dstip\">\n");
	groups_enum(GROUPS_ALL,listif);
	printf("</SELECT>\n");

	/* DST port */
	printf("<STRONG>Ports:</STRONG>\n");
	printf("<SELECT NAME=\"dstport\">\n");
	ports_enum(PORTS_ALL,listport);
	printf("</SELECT>\n");

	/* ICMP Codes */
	printf("<STRONG>ICMP Code:</STRONG>\n");
	printf("<SELECT NAME=\"icmpcode\">\n");
	icmp_codeenum(listicmp);
	printf("</SELECT>\n");
	printf("<BR>\n");


	printf("<INPUT TYPE=\"submit\" VALUE=\"ADD RULE\">\n");
	printf("<INPUT TYPE=\"hidden\" NAME=\"ruleset\" VALUE=\"%s\">\n",name);
	printf("</FORM>\n");

	/* Delete and Edit only if there are any rules */
	if( (j=rules_num(type))>0 ){
		/* Delete rule */
		printf("<FORM ACTION=\"%s\" METHOD=POST>\n",QUERY_DELETE_RULE);
		printf("<SELECT NAME=\"selected_rule\">\n");
		pos=0;
		rules_enum(type,listrule);
		printf("</SELECT>\n");
		printf("<INPUT TYPE=\"submit\" VALUE=\"DELETE RULE\">\n");
		printf("<INPUT TYPE=\"hidden\" NAME=\"ruleset\" VALUE=\"%s\">\n",name);
		printf("</FORM>\n");

		/* Edit rule */
		printf("<FORM ACTION=\"%s\" METHOD=POST>\n",QUERY_EDIT_RULE);
		printf("<SELECT NAME=\"selected_rule\">\n");
		pos=0;
		rules_enum(type,listrule);
		printf("</SELECT>\n");
		printf("<INPUT TYPE=\"submit\" VALUE=\"EDIT RULE\">\n");
		printf("<INPUT TYPE=\"hidden\" NAME=\"ruleset\" VALUE=\"%s\">\n",name);
		printf("</FORM>\n");
	}

	/* table of all rules */
	printf("<TABLE BORDER>\n");
	printf("<CAPTION><STRONG>Current %s Rules</STRONG></CAPTION>\n",Name);
	printf("<TR>\n");
	printf("<TH> # </TH> <TH> Action </TH> <TH> Protocol </TH> <TH> Iface </TH> <TH> Src-IP </TH> <TH> Scr-port </TH> <TH> Dst-IP </TH> <TH> Dst-port </TH> <TH> ICMP Type </TH> <TH> ICMP Code </TH>\n");
	printf("</TR>\n");
	pos=0;
	rules_enum(type,ruletable);
	printf("</TABLE>\n");

	printf("</BODY></HTML>\n");
	return(0);
}

